"""
Scraper for the Supreme Court of Tennessee
CourtID: tenn
Court Short Name: Tenn.
"""
from juriscraper.OpinionSite import OpinionSite
import time
from datetime import date
from lxml import html


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = 'http://www.tsc.state.tn.us/courts/supreme-court/opinions'
        self.court_id = self.__module__
        self.back_scrape_iterable = range(0, 131)

    def _get_download_urls(self):
        return [t for t in self.html.xpath("//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/a/@href")]

    def _get_case_names(self):
        return [t for t in self.html.xpath("//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/a/text()")]

    def _get_lower_courts(self):
        return [t.strip() for t in self.html.xpath("//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td[2]/text()")]

    def _get_case_dates(self):
        dates = []
        for s in self.html.xpath("//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td[4]/span/text()"):
            dates.append(date.fromtimestamp(time.mktime(time.strptime(s, '%m/%d/%y'))))
        return dates

    def _get_docket_numbers(self):
        return [t for t in self.html.xpath("//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/div[./@class[contains(.,'number')]]/text()")]

    # Here we are using the 'judges' field to list the Authoring Judge.
    # It would be better for AbstractSite to have an 'author' field and to put
    # these there.
    def _get_judges(self):
        judges = []
        for t in self.html.xpath("//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/div/text()"):
            if 'Authoring' in t:
                # We strip the text 'Authoring Judge: '
                judges.append(t[17:])
            else:
                continue
        return judges

    def _get_lower_court_judges(self):
        trial_judges = []
        for t in self.html.xpath("//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/div/text()"):
            if 'Trial' in t:
                # We strip the text 'Trial Court Judge: '
                trial_judges.append(t[19:])
            else:
                continue
        return trial_judges

    def _get_summaries(self):
        summaries = []
        for e in self.html.xpath("//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td//div[@class='case-body']"):
            ps = e.xpath('p')
            s = ''
            for p in ps:
                s += html.tostring(p, method='text', encoding='unicode')
            summaries.append(s)
        return summaries

    def _get_precedential_statuses(self):
        return ['Published'] * len(self.case_names)

    def _download_backwards(self, page):
        self.url = '%s?page=%s' % (self.url, page)
        self.html = self._download()




import time
from datetime import date

from juriscraper.AbstractSite import logger
from juriscraper.OpinionSite import OpinionSite
import certifi
from lxml import html
import requests


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.base_url = 'http://digitalcommons.law.villanova.edu/thirdcircuit_{year}/'
        self.url = self.base_url.format(
            year=date.today().year
        )
        self.court_id = self.__module__
        self.back_scrape_iterable = range(2003, date.today().year + 1)
        self.case_xpath = "//p[@class='article-listing']/a/@href"
        self.next_page_xpath = "//div[@class='adjacent-pagination']//li[@class='active']//following-sibling::li/a/@href"

    def _download(self, request_dict={}):
        if self.test_mode_enabled():
            # Note that this is returning a list of HTML trees.
            html_trees = [
                super(Site, self)._download(request_dict=request_dict),
            ]
        else:
            html_l = super(Site, self)._download(request_dict)
            html_trees = self._get_case_html_page([], html_l, request_dict)

            # handle the next page data
            if html_l.xpath(self.next_page_xpath):
                self.url = html_l.xpath(self.next_page_xpath)[0]
                html_next_trees = self._download(request_dict)
                html_trees.extend(html_next_trees)
        return html_trees

    def _get_case_html_page(self, html_trees, html_l, request_dict):
        """Gets each of the individual case pages"""
        s = requests.session()
        for case_url in html_l.xpath(self.case_xpath):
            logger.info("  Getting sub-page at: %s" % case_url)
            r = s.get(
                case_url,
                headers={'User-Agent': 'Juriscraper'},
                verify=certifi.where(),
                timeout=60,
                **request_dict
            )

            r.raise_for_status()

            # If the encoding is iso-8859-1, switch it to cp1252 (a superset)
            if r.encoding == 'ISO-8859-1':
                r.encoding = 'cp1252'

            # Grab the content
            text = self._clean_text(r.text)
            html_tree = html.fromstring(text)
            html_tree.make_links_absolute(self.url)

            remove_anchors = lambda url: url.split('#')[0]
            html_tree.rewrite_links(remove_anchors)
            html_trees.append(html_tree)
        return html_trees

    def _get_case_names(self):
        data = []
        for html_tree in self.html:
            try:
                data.append(html_tree.xpath("//div[@id='title']//a/text()")[0])
            except IndexError:
                data.append('')
        return data

    def _get_download_urls(self):
        data = []
        for html_tree in self.html:
            try:
                data.append(html_tree.xpath("//div[@id='title']//a/@href")[0])
            except IndexError:
                data.append('')
        return data

    def _get_case_dates(self):
        data = []
        for html_tree in self.html:
            try:
                date__ = html_tree.xpath("//div[@id='publication_date']/p/text()")[0]
                data.append(date.fromtimestamp(time.mktime(time.strptime(date__, '%m-%d-%Y'))))
            except IndexError:
                data.append('')
        return data

    def _get_docket_numbers(self):
        data = []
        for html_tree in self.html:
            try:
                data.append(html_tree.xpath("//div[@id='comments']//p/text()")[0].replace('No.', '').strip())
            except IndexError:
                data.append('')
        return data

    def _get_precedential_statuses(self):

        data = []
        for html_tree in self.html:
            try:
                status = html_tree.xpath("//div[@id='precedent']//p/text()")[0]
                if 'non' in status.lower():
                    data.append('Unpublished')
                else:
                    data.append('Published')
            except IndexError:
                data.append('Unknown')
        return data

    def _get_lower_courts(self):
        data = []
        for html_tree in self.html:
            try:
                data.append(html_tree.xpath("//div[@id='abstract']//p/text()")[0])
            except IndexError:
                data.append('')
        return data

    def _download_backwards(self, d):
        self.url = self.base_url.format(year=d)
        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200

    def _post_parse(self):
        """This will remove the cases without a case name"""
        to_be_removed = [index for index, case_name in
                         enumerate(self.case_names)
                         if not case_name]

        for attr in self._all_attrs:
            item = getattr(self, attr)
            if item is not None:
                new_item = self.remove_elements(item, to_be_removed)
                self.__setattr__(attr, new_item)

    @staticmethod
    def remove_elements(list_, indexes_to_be_removed):
        return [i for j, i in enumerate(list_) if j not in indexes_to_be_removed]



"""Scraper for Minnesota Attorney General Opinions
CourtID: minnag
Court Short Name: MN
Author: David Cook
Reviewer:
"""

import re

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.domain = 'http://www.ag.state.mn.us'
        self.url = 'http://www.ag.state.mn.us/office/opinions/DATE.asp'
        self.opinions = []

    def _download(self, request_dict={}):
        html = super(Site, self)._download(request_dict)
        self._extract_case_data_from_html(html)
        return html

    def _extract_case_data_from_html(self, html):
        for p in html.xpath("//div[@id='content']/div[@id='op']/p"):
            text = p.xpath('a/strong/following-sibling::text()[1]')[0]
            name = ' '.join(text.split()[1:])  # strip leading dash charater, which can be one of various unicode dash characters
            name = re.sub("\\s+", " ", name).strip()
            summary = ' '.join(p.xpath('br/following-sibling::text() | br/following-sibling::*//text()'))
            summary = re.sub('\\s+', ' ', summary).strip()

            self.opinions.append({
                'name': name,
                'url': p.xpath('a/@href')[0],
                'date': convert_date_string(p.xpath('a/strong/text()')[0]),
                'summary': summary
            })

    def _get_case_names(self):
        return [opinion['name'] for opinion in self.opinions]

    def _get_download_urls(self):
        return [opinion['url'] for opinion in self.opinions]

    def _get_case_dates(self):
        return [opinion['date'] for opinion in self.opinions]

    def _get_summaries(self):
        return [opinion['summary'] for opinion in self.opinions]

    def _get_precedential_statuses(self):
        return ['Published'] * len(self.opinions)
